/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.site.admin.web.internal.display.context;

import com.liferay.petra.string.StringPool;
import com.liferay.portal.kernel.model.LayoutSetPrototype;
import com.liferay.site.admin.web.internal.constants.SiteAdminConstants;
import com.liferay.site.initializer.SiteInitializer;

import java.util.Locale;

/**
 * @author Marco Leo
 */
public class SiteInitializerItemDisplayContext {

	public SiteInitializerItemDisplayContext(
		LayoutSetPrototype layoutSetPrototype, Locale locale) {

		_siteInitializerKey = StringPool.BLANK;
		_icon = StringPool.BLANK;
		_layoutSetPrototypeId = layoutSetPrototype.getLayoutSetPrototypeId();
		_name = layoutSetPrototype.getName(locale);
		_type = SiteAdminConstants.CREATION_TYPE_SITE_TEMPLATE;
	}

	public SiteInitializerItemDisplayContext(
		SiteInitializer siteInitializer, Locale locale) {

		_siteInitializerKey = siteInitializer.getKey();
		_icon = siteInitializer.getThumbnailSrc();
		_layoutSetPrototypeId = 0;
		_name = siteInitializer.getName(locale);
		_type = SiteAdminConstants.CREATION_TYPE_INITIALIZER;
	}

	public String getIcon() {
		return _icon;
	}

	public String getKey() {
		if (isCreationTypeInitializer()) {
			return _siteInitializerKey;
		}

		return String.valueOf(_layoutSetPrototypeId);
	}

	public long getLayoutSetPrototypeId() {
		return _layoutSetPrototypeId;
	}

	public String getName() {
		return _name;
	}

	public String getSiteInitializerKey() {
		return _siteInitializerKey;
	}

	public String getType() {
		return _type;
	}

	public boolean isCreationTypeInitializer() {
		if (_type.equals(SiteAdminConstants.CREATION_TYPE_INITIALIZER)) {
			return true;
		}

		return false;
	}

	public boolean isCreationTypeSiteTemplate() {
		return !isCreationTypeInitializer();
	}

	private final String _icon;
	private final long _layoutSetPrototypeId;
	private final String _name;
	private final String _siteInitializerKey;
	private final String _type;

}